@@ -74,7 +74,7 @@ WAIT_STATUS_READING = 1
 WAIT_STATUS_WRITING = 2
 WAIT_STATUS_READWRITING = WAIT_STATUS_READING | WAIT_STATUS_WRITING
 
-BUF_SIZE = 8 * 1024
+BUF_SIZE = 32 * 1024
 
 
 class TCPRelayHandler(object):
@@ -433,7 +433,7 @@ class TCPRelayHandler(object):
                 self._on_remote_error()
             if self._stage == STAGE_DESTROYED:
                 return
-            if event & eventloop.POLL_IN:
+            if event & (eventloop.POLL_IN | eventloop.POLL_HUP):
                 self._on_remote_read()
             if self._stage == STAGE_DESTROYED:
                 return
@@ -444,7 +444,7 @@ class TCPRelayHandler(object):
                 self._on_local_error()
             if self._stage == STAGE_DESTROYED:
                 return
-            if event & eventloop.POLL_IN:
+            if event & (eventloop.POLL_IN | eventloop.POLL_HUP):
                 self._on_local_read()
             if self._stage == STAGE_DESTROYED:
                 return
